"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

Object subclass:#Project
	instanceVariableNames:'name changeSet views directoryName properties packageName
		repositoryDirectory repositoryModule overwrittenMethods
		subProjects prerequisites isLoaded'
	classVariableNames:'CurrentProject SystemProject NextSequential AllProjects
		LoadedProjects'
	poolDictionaries:''
	category:'System-Support'
!

Object subclass:#MethodInfo
	instanceVariableNames:'conditionForInclusion methodName className fileName'
	classVariableNames:''
	poolDictionaries:''
	privateIn:Project
!

Object subclass:#ClassInfo
	instanceVariableNames:'conditionForInclusion className classFileName'
	classVariableNames:''
	poolDictionaries:''
	privateIn:Project
!

!Project class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    this class is still under construction (especially the build features are unfinished).
    Currently, all it does is keep track of per-project views 
    (to hide or show them), define the directory when filing-out,
    and define packageNames for new classes and methods.

    instance variables:
	name            <String>        the name of this project, as shown
					in a ProjectView

	changeSet       <ChangeSet>     changes done, while this was the active project

	views           <Collection>    views opened while this was the active project

	directoryName   <String>        directory name, where fileOuts are done

	properties 

	packageName     <String>        given to classes/methods which are created while
					this is the active project

	repositoryDirectory             (default) name of the repository, when a new source containers are
					created.

	repositoryModule                (default) name of the module, when new source containers are
					created.

    Future: 
	- keep track of per-project changes
	- allow speficiation of the type of the project (application or library)
	- allow building of whatever the target (as defined by the type) is
	  (this will allow build of class libs and apps by clicking a button)
	- allow removal of project specific classes, methods etc.

    [author:]
	Claus Gittinger
"
! !

!Project class methodsFor:'initialization'!

initKnownProjects
    "this is a temporary experimental kludge -
     once the ProjectBrowser is finished, this info is read from
     '.prj' files ..."

    "/ collect project info while scanning all classes
    "/ and methods ...

    |packages anyUnloaded|

    packages := Dictionary new.

    AllProjects isNil ifTrue:[
	AllProjects := OrderedCollection new.
    ] ifFalse:[
	AllProjects do:[:p | packages at:p package asSymbol put:p].
    ].
    Smalltalk allClassesDo:[:aClass |
	|packageID prj classFilename pkgInfo revInfo 
	 repositoryPath dir module lib nm|

	aClass isMeta ifFalse:[
	    (aClass isNamespace not or:[aClass == Smalltalk]) ifTrue:[
		packageID := aClass package asSymbol.
		(packages includesKey:packageID) ifFalse:[

		    "/ a new one ...

		    prj := self new.
		    prj package:packageID.
		    prj directory:'???'.
		    prj repositoryModule:'unknown'.
		    prj repositoryDirectory:'unknown'.

		    nm := 'unknown'.

		    pkgInfo := aClass packageSourceCodeInfo.
		    pkgInfo notNil ifTrue:[
			module := pkgInfo at:#module ifAbsent:nil.
			module notNil ifTrue:[
			    prj repositoryModule:module    
			].
			dir := pkgInfo at:#directory ifAbsent:nil.
			dir notNil ifTrue:[
			    prj repositoryDirectory:dir    
			].
			lib := pkgInfo at:#library ifAbsent:nil.
			lib notNil ifTrue:[
			    prj type:#library.
			].
			prj isLoaded:true.
		    ].

		    nm := (module ? 'unknown')
			  , ':'
			  , (dir ? (lib ? 'unknown')).
			
		    prj name:nm.
		    AllProjects add:prj.
		    packages at:packageID put:prj.
		].
	    ].
	].
    ].
    Method allSubInstancesDo:[:aMethod |
	|packageID prj who mthdClass|

	packageID := aMethod package asSymbol.
	(packages includesKey:packageID) ifFalse:[
	    who := aMethod who.
	    who notNil ifTrue:[ "/ skip unbound methods ...
		"/ a new one ...
		prj := self new.
	"/            prj name:libName.
		prj package:packageID.
		prj type:#library.
		prj directory:'???'.
		prj repositoryModule:'stx'.
		prj repositoryDirectory:'???'.
		prj isLoaded:true.
		AllProjects add:prj.
		packages at:packageID put:prj.
	    ]
	]
    ].

    "/ walk over binary modules, to find out directory names ...

    ObjectFileLoader loadedObjectHandles do:[:h |
	|cls prj mDir|

	cls := h classes firstIfEmpty:nil.
	cls notNil ifTrue:[
	    prj := packages at:cls package ifAbsent:nil.
	    prj notNil ifTrue:[
		mDir := h pathName asFilename directory pathName.
		prj directory = '???' ifTrue:[
		    prj directory:mDir
		] ifFalse:[
		    prj directory ~= mDir ifTrue:[
			('Project [warning]: conflicting project directories for ' , cls package) infoPrintCR.
		    ]
		]
	    ]
	].
    ].

    self changed:#allProjects

"/    |stx p|
"/
"/    stx := self new name:'stx'.
"/    stx packageName:'noPackage'.
"/    stx changeSet:nil.
"/    stx type:#smalltalk.
"/    stx comment:'The ST/X project itself'.
"/
"/    AllProjects add:stx.
"/
"/    #(
"/        ('libbasic'      #'stx:libbasic'            'Basic (non-GUI) classes. Required for all applications')
"/        ('libbasic2'     #'stx:libbasic2'           'More basic (non-GUI) classes. Required for most applications')
"/        ('libbasic3'     #'stx:libbasic3'           'More basic (non-GUI) classes. Required for development')
"/        ('libcomp'       #'stx:libcomp'             'The bytecode compiler. Required for all applications')
"/        ('libview'       #'stx:libview'             'Low level GUI classes. Required for all GUI applications')
"/        ('libview2'      #'stx:libview2'            'Additional low level GUI classes. Required for most GUI applications')
"/        ('libwidg'       #'stx:libwidg'             'Basic widgets. Required for all GUI applications')
"/        ('libwidg2'      #'stx:libwidg2'            'More widgets. Required for most GUI applications')
"/        ('libwidg3'      #'stx:libwidg3'            'More (fun) widgets. Seldom required')
"/        ('libtool'       #'stx:libtool'             'Development applications. Required for program development')
"/        ('libtool2'      #'stx:libtool2'            'More development applications. Required for GUI development')
"/        ('libui'         #'stx:libui'               'UI spec classes. Required for UIPainter applications')
"/        ('libhtml'       #'stx:libhtml'             'HTML related classes. Required for Web applications and the HTML browser')
"/        ('libodbc'       #'stx:libodbc'             'ODBC interface classes.')
"/        ('libopengl'     #'stx:libopengl'           'OpenGL interface classes.')
"/        ('persistency'   #'stx:goodies/persistency' 'Simple DB interface')
"/        ('goodies'       #'stx:goodies'             'Misc goodies - not really maintained')
"/    ) do:[:entry |
"/        |libName package comment|
"/
"/        libName := entry at:1.
"/        package := entry at:2.
"/        comment := entry at:3.
"/
"/        p := self new name:libName.
"/        p packageName:package.
"/        p type:#library.
"/        p comment:comment.
"/        stx addSubProject:p.
"/    ].

    "
     AllProjects := nil.
     self initKnownProjects
    "

    "Modified: / 23.3.1999 / 14:20:12 / cg"
!

initialize
    SystemProject isNil ifTrue:[
	NextSequential := 1.
	SystemProject := self new name:'default'.
	SystemProject package:'private'.
	SystemProject defaultNameSpace:Smalltalk.
	SystemProject comment:'A default (dummy) project. 
Will be made the current project in case no real project is ever activated.'.

	"
	 the SystemProject does not keep a record if changes,
	 but instead depends on the changes file - recording anything there.
	"
	SystemProject changeSet:nil.
    ].

    CurrentProject := SystemProject.
    AllProjects := nil.
"/ now done lazy ...
"/    self initKnownProjects.

    "
     SystemProject := nil.
     Project initialize
    "
! !

!Project class methodsFor:'instance creation'!

new
    ^ self basicNew initialize
! !

!Project class methodsFor:'accessing'!

addKnownProject:aProject
    |allProjects|

    allProjects := self knownProjects.
    (allProjects detect:[:p | p package = aProject package] ifNone:nil) notNil ifTrue:[
	self warn:'Project for ' , aProject package , ' is already present.'.
	^ self.
    ].

    AllProjects add:aProject.
    self changed:#allProjects
!

addLoadedProject:aProject
    aProject isLoaded:true.
    self addKnownProject:aProject.
!

current
    "return the currently active project"

    ^ CurrentProject

    "
     Project current
    "
!

current:aProject
    "set the currently active project"

    CurrentProject := aProject.
    self changed:#currentProject
!

currentPackageName
    CurrentProject notNil ifTrue:[
	^ CurrentProject package
    ].
    ^ nil

    "
     Project currentPackageName
    "
!

defaultProject
    "return the SystemDefault project"

    ^ SystemProject.
!

knownProjects
    AllProjects isNil ifTrue:[
	self initKnownProjects
    ].
    ^ AllProjects ? #()
!

loadedProjects
    ^ self knownProjects select:[:p | p isLoaded]
!

projectNamed:aProjectName
    "retrieve the named project; return nil if not known"

    ^ self knownProjects detect:[:p | p name = aProjectName] ifNone:nil.

    "
     Project projectNamed:'libbasic'
    "
!

projectWithId:aPackageId
    "retrieve the project with a particular id; return nil if not known"

    ^ self knownProjects detect:[:p | p package = aPackageId] ifNone:nil.

    "
     Project projectWithId:#'stx:libbasic'
    "
!

setDefaultProject
    "set the currently active project to be the SystemDEfault project"

    self current:SystemProject.
!

setProject:aProjectOrNil
    "set the currently active project without updating others"

    CurrentProject := aProjectOrNil.

    "Created: 7.2.1996 / 14:00:45 / cg"
    "Modified: 7.2.1996 / 14:01:16 / cg"
! !

!Project class methodsFor:'changes management'!

addClassDefinitionChangeFor:aClass
    "add a class-def-change for aClass to the current project"

    |p c|

    p := CurrentProject.
    (p notNil 
    and:[(c := p changeSet) notNil]) ifTrue:[
	c addClassDefinitionChangeFor:aClass 
    ]

    "Created: 3.12.1995 / 13:44:58 / cg"
    "Modified: 3.12.1995 / 13:58:04 / cg"
!

addMethodCategoryChange:aMethod category:newCategory in:aClass
    "add a method-category-change for aMethod in aClass to the current project"

    |p c|

    p := CurrentProject.
    (p notNil 
    and:[(c := p changeSet) notNil]) ifTrue:[
	c addMethodCategoryChange:aMethod category:newCategory in:aClass 
    ]
!

addMethodChange:aMethod in:aClass
    "add a method change in aClass to the current project"

    |p c|

    p := CurrentProject.
    (p notNil 
    and:[(c := p changeSet) notNil]) ifTrue:[
	c addMethodChange:aMethod in:aClass 
    ]
!

addMethodPrivacyChange:aMethod in:aClass
    "add a privacy change for aMethod in aClass to the current project"

    |p c|

    p := CurrentProject.
    (p notNil 
    and:[(c := p changeSet) notNil]) ifTrue:[
	c addMethodPrivacyChange:aMethod in:aClass 
    ]

    "Modified: 27.8.1995 / 22:48:17 / claus"
!

addPrimitiveDefinitionsChangeFor:aClass
    "add a primitiveDef change for aClass to the current project"

    |p c|

    p := CurrentProject.
    (p notNil 
    and:[(c := p changeSet) notNil]) ifTrue:[
	c addPrimitiveDefinitionsChangeFor:aClass 
    ]
!

addPrimitiveFunctionsChangeFor:aClass
    "add a primitiveFuncs change for aClass to the current project"

    |p c|

    p := CurrentProject.
    (p notNil 
    and:[(c := p changeSet) notNil]) ifTrue:[
	c addPrimitiveFunctionsChangeFor:aClass 
    ]
!

addPrimitiveVariablesChangeFor:aClass
    "add a primitiveVars change for aClass to the current project"

    |p c|

    p := CurrentProject.
    (p notNil 
    and:[(c := p changeSet) notNil]) ifTrue:[
	c addPrimitiveVariablesChangeFor:aClass 
    ]
!

addRemoveSelectorChange:aSelector in:aClass
    "add a method-remove change in aClass to the current project"

    |p c|

    p := CurrentProject.
    (p notNil 
    and:[(c := p changeSet) notNil]) ifTrue:[
	c addRemoveSelectorChange:aSelector in:aClass 
    ]

    "Created: / 16.2.1998 / 12:45:10 / cg"
!

currentProjectDirectory
    "return the name of the directory to use for fileOut.
     The returned name already includes a file-separator at the end, 
     so the filename can be concatenated to it."

    |p dirName|

    p := CurrentProject.
    p notNil ifTrue:[
	dirName := p directory  
    ] ifFalse:[
	dirName := Filename currentDirectory name
    ].
    ^ dirName

    "Modified: 7.9.1997 / 23:52:25 / cg"
!

rememberOverwrittenMethod:newMethod from:oldMethod
    "remember a method (from another package) being overwritten.
     This is only remembered, if the current project is not the
     system project (to avoid filling this remembered-table)"

    |p|

    p := CurrentProject.
    (p notNil 
    and:[p ~~ SystemProject]) ifTrue:[
	p rememberOverwrittenMethod:newMethod from:oldMethod
    ] ifFalse:[
"/        'Project [info]: DefaultProject does not remember overwritten methods' infoPrintCR
    ]

    "Modified: / 7.3.1998 / 13:38:39 / cg"
! !

!Project methodsFor:'accessing'!

addSubProject:aProject
    "add a subproject - obsolete; we use prerequisites now"

    subProjects isNil ifTrue:[
	subProjects := OrderedCollection new.
    ].
    subProjects add:aProject
!

changeSet
    "return the set of changes made in this project"

    ^ changeSet

    "Modified: 27.1.1997 / 11:58:36 / cg"
!

changeSet:aChangeSet
    "set the set of changes made in this project - dangerous, you may loose
     the actual changeSet."

    changeSet := aChangeSet

    "Modified: 27.1.1997 / 11:59:02 / cg"
!

defaultNameSpace
    "return the defaultNameSpace of this project.
     New classes will (if not specified by a directive) be installed
     in this nameSpace. Useful, when filing in ST-80 code, to avoid
     overwriting of standard classes."

    ^ (self propertyAt:#defaultNameSpace) ? Smalltalk

!

defaultNameSpace:aNamespace
    "set the defaultNameSpace of this project.
     New classes will (if not specified by a directive) be installed
     in this nameSpace. Useful, when filing in ST-80 code, to avoid
     overwriting of standard classes."

    |prevDefault|

    prevDefault := self defaultNameSpace.
    aNamespace ~~ prevDefault ifTrue:[
	self propertyAt:#defaultNameSpace put:aNamespace.
	self changed:#defaultNameSpace.
	self == CurrentProject ifTrue:[
	    Project changed:#defaultNameSpace 
	]
    ]

    "Created: 2.1.1997 / 19:54:37 / cg"
    "Modified: 27.1.1997 / 12:00:01 / cg"
!

directory
    "return the projects directory.
     If not specified, a fileOut will be done into that directory"

    directoryName isNil ifTrue:[^ '.'].
    ^ directoryName

    "Modified: 27.1.1997 / 12:00:41 / cg"
!

directory:aDirectoryName
    "set the projects directory.
     If not specified, a fileOut will be done into that directory"

    directoryName := aDirectoryName.
    self changed:#directory.
    self == CurrentProject ifTrue:[
	Project changed:#directory 
    ]

    "Modified: 27.1.1997 / 12:00:47 / cg"
!

isLoaded:aBoolean
    isLoaded := aBoolean
!

libraryName
    "return the projects library name.
     This is the name of a classLibrary, created from this project"

    |nm|

    nm := self name.
    ^ nm copyFrom:(nm lastIndexOfAny:':/')+1

"/    ^ (self name copyReplaceAll:$/ with:$_) replaceAll:$: with:$_
!

name
    "return the projects name.
     This is for the user only - shown in the projectViews label"

    ^ name

    "Modified: 27.1.1997 / 12:01:16 / cg"
!

name:aString
    "set the projects name.
     This is for the user only - shown in the projectViews label"

    name := aString.
    self changed:#name.
    self == CurrentProject ifTrue:[
	Project changed:#name
    ]

    "Modified: 27.1.1997 / 12:01:09 / cg"
!

overwrittenMethods
    "return the set of methods which were overwritten in this project.
     This information allows uninstalling, by switching back to the
     original methods."

    ^ overwrittenMethods

    "Created: 27.1.1997 / 11:57:21 / cg"
    "Modified: 27.1.1997 / 12:09:14 / cg"
!

package
    "return the projects package identifier.
     This identifier marks all methods and new classes which were created
     in this project."

    ^ packageName

    "Modified: 27.1.1997 / 12:10:00 / cg"
!

package:aPackageId
    "set the projects package identifier.
     This identifier marks all methods and new classes which were created
     in this project."

    packageName := aPackageId

    "Modified: 27.1.1997 / 12:10:00 / cg"
!

packageName:aPackageId
    "set the projects package identifier.
     This identifier marks all methods and new classes which were created
     in this project."

    self package:aPackageId

    "Modified: 27.1.1997 / 12:10:00 / cg"
!

prerequisiteClasses
    "return the prerequisiteClasses of the project"

    ^ (self propertyAt:#prerequisiteClasses) ? #()
!

prerequisiteClasses:aCollectionOfClassesOrClassNames
    "set the prerequisiteClasses of the project"

    ^ self propertyAt:#prerequisiteClasses put:aCollectionOfClassesOrClassNames

    "Modified: / 26.4.1999 / 23:33:44 / cg"
!

prerequisitePackages
    "return the prerequisitePackages of the project"

    ^ self prerequisites

    "Created: / 26.4.1999 / 23:33:22 / cg"
!

prerequisitePackages:aCollectionOfPackagesOrPackageNames
    "set the prerequisitePackages of the project"

    ^ self prerequisites:aCollectionOfPackagesOrPackageNames

    "Modified: / 26.4.1999 / 23:34:40 / cg"
!

prerequisites
    "return the prerequisites of the project"

    ^ prerequisites ? #()
!

prerequisites:aCollectionOfProjects
    "set the prerequisites of the project"

    prerequisites := aCollectionOfProjects
!

repositoryDirectory
    "return the projects default repository location.
     This is offered initially, when classes are checked into the
     source repository initially"

    ^ repositoryDirectory

    "Created: 25.11.1995 / 18:04:51 / cg"
    "Modified: 27.1.1997 / 12:13:35 / cg"
!

repositoryDirectory:aRelativePathName
    "set the projects default repository location.
     This will be offered initially, when classes are checked into the
     source repository initially"

    repositoryDirectory := aRelativePathName

    "Created: 25.11.1995 / 18:05:06 / cg"
    "Modified: 27.1.1997 / 12:13:28 / cg"
!

repositoryModule
    "return the projects default repository module name.
     This is offered initially, when classes are checked into the
     source repository initially"

    ^ repositoryModule

    "Created: 25.11.1995 / 18:04:51 / cg"
    "Modified: 27.1.1997 / 12:13:50 / cg"
!

repositoryModule:aString
    "set the projects default repository module name.
     This is offered initially, when classes are checked into the
     source repository initially"

    repositoryModule := aString

    "Created: 25.11.1995 / 18:05:06 / cg"
    "Modified: 27.1.1997 / 12:13:57 / cg"
!

subProjects
    "return the subprojects - obsolete; we use prerequisites now"

    ^ subProjects ? #()
!

views
    "return a collection of views which were opened in this project"

    ^ views asArray

    "Modified: 27.1.1997 / 12:14:18 / cg"
!

views:aSetOfViews
    "set the collection of views which were opened in this project"

    views := WeakIdentitySet withAll:aSetOfViews

    "Modified: 27.1.1997 / 12:14:26 / cg"
! !

!Project methodsFor:'administration'!

removeClassesFromSystem
    "remove the all of my classes & patches from the system"

    self classInfo do:[:clsInfo |
	|clsName cls|

	clsName := clsInfo className.
	clsName isSymbol ifTrue:[
	    cls := Smalltalk at:clsName.
	    cls notNil ifTrue:[
		cls removeFromSystem.
	    ].
	] ifFalse:[
	    self halt
	].
    ].

!

removeFromSystem
    "remove the project and all of its classes & patches from the
     system"

    self removeClassesFromSystem.
    AllProjects remove:self ifAbsent:nil.
! !

!Project methodsFor:'changes'!

rememberOverwrittenMethod:newMethod from:oldMethod
    "this is sent whenever a method is installed, which overwrites
     an existing method from a different package.
     Allows previous methods to be reconstructed."

    overwrittenMethods isNil ifTrue:[
	overwrittenMethods := IdentityDictionary new.
    ].
    overwrittenMethods at:newMethod put:oldMethod

    "Created: 27.1.1997 / 11:52:01 / cg"
    "Modified: 30.1.1997 / 21:10:51 / cg"
! !

!Project methodsFor:'initialization'!

initialize
    |numString|

    views := WeakIdentitySet new.
    numString := NextSequential printString.
    NextSequential := NextSequential + 1.

    name := 'new Project-' , numString.
    packageName := 'private-' , numString.

    "/
    "/ for tiny-configurations, allow ChangeSet to be absent
    "/
    ChangeSet notNil ifTrue:[
	changeSet := ChangeSet new.
    ].
    self directory:'.'.
    self repositoryModule:(OperatingSystem getLoginName).
    self repositoryDirectory:'private'

    "Created: 25.11.1995 / 18:05:44 / cg"
    "Modified: 3.1.1997 / 13:24:10 / cg"
! !

!Project methodsFor:'load & save'!

conditionForAutoloadIsTrue:cond
    "check condition ..."

    ^ cond == #autoload 
!

conditionForInclusionIsTrue:cond
    "check condition ..."

    cond == #always ifTrue:[^ true].
    cond == #autoload ifTrue:[^ true].
    cond == #never ifTrue:[^ false].

    cond == #unix ifTrue:[
	^ OperatingSystem isUNIXlike
    ].
    cond == #win32 ifTrue:[
	^ OperatingSystem isMSWINDOWSlike
    ].
    cond == #vms ifTrue:[
	^ OperatingSystem isVMSlike
    ].
    cond == #macos ifTrue:[
	^ OperatingSystem isMAClike
    ].
    self halt:'bad condition'
!

loadClassesFromProjectDirectory
    "load my classes into the system.
     The classes are loaded from the directory as defined by my
     directory insTvar"

    |myDirectory firstTrip|

    myDirectory := self directory asFilename.
    firstTrip := true.

    "/ read twice; if the load order was not correct,
    "/ this will fix things (i.e. nil superclasses ...)

    2 timesRepeat:[
	self classInfo do:[:clsInfo |
	    |clsName clsFileNameString cls clsFilename 
	     cond include asAutoload|

	    clsName := clsInfo className.
	    clsFileNameString := clsInfo classFileName.
	    clsFileNameString isNil ifTrue:[clsFileNameString := clsName , '.st'].

	    clsName isSymbol ifTrue:[
		clsFilename := myDirectory construct:clsFileNameString.
		cls := Smalltalk at:clsName.
		(cls isNil or:[firstTrip]) ifTrue:[
		    "/ ok - really not yet loaded.
		    Transcript showCR:'loading ' , clsFilename pathName , ' ...'.
		] ifFalse:[
		    Transcript showCR:'reloading ' , clsFilename pathName , ' ...'.
		].
		"/ check condition ...
		cond := clsInfo conditionForInclusion.
		(self conditionForInclusionIsTrue:cond) ifTrue:[
		    (self conditionForAutoloadIsTrue:cond) ifTrue:[
			Smalltalk
			    installAutoloadedClassNamed:clsName
			    category:'autoloaded'
			    package:self package    
			    revision:nil
		    ] ifFalse:[
			Smalltalk fileIn:clsFilename
		    ]
		]
	    ] ifFalse:[
		self halt
	    ].
	].

	firstTrip := false.
    ].

!

loadFromProjectFile:aFilename
    "fill all of my attributes from a projects file (.prj-file)"

    |f l pack targetConditions s sourcesSubDir files module methodsFile|

    f := aFilename asFilename.
    directoryName := f directory pathName.
    self directory:(f directory pathName).
    pack := ResourcePack fromFile:f baseName directory:directoryName.

    "/ convert the resourcePack ...

    packageName := pack at:'package' ifAbsent:packageName.
    name := pack at:'name' ifAbsent:name.

    self type:(pack at:'type' ifAbsent:#application) asSymbol.

    module := pack at:'repository.module' ifAbsent:nil.
    module notNil ifTrue:[
	repositoryModule := module.
	repositoryDirectory := pack at:'repository.directory' ifAbsent:''.
    ].

    prerequisites := pack at:'prerequisites' ifAbsent:#().
    self prerequisiteClasses:(pack at:'prerequisiteClasses' ifAbsent:#()).

    s := pack at:'nameSpace' ifAbsent:nil.
    s notNil ifTrue:[
	self defaultNameSpace:(Namespace name:s asSymbol).
    ].

    subProjects := pack at:'subProjects' ifAbsent:subProjects.
    (s := pack at:'comment' ifAbsent:nil) notNil ifTrue:[
	self comment:s
    ].

    "/ first, all of the conditions ...
    targetConditions := Dictionary new.
    pack keysAndValuesDo:[:key :val |
	|conditionKey|

	(key startsWith:'target.condition.') ifTrue:[
	    conditionKey := key copyFrom:'target.condition.' size + 1.
	    targetConditions at:conditionKey put:val.
	]
    ].

    properties isNil ifTrue:[
	properties := IdentityDictionary new
    ].
    properties at:#'targetconditions' put:targetConditions.

    sourcesSubDir := pack at:'sources' ifAbsent:nil.
    sourcesSubDir notNil ifTrue:[
	properties at:#'sourcesDirectory' put:sourcesSubDir.
    ].
    methodsFile := pack at:'methodsFile' ifAbsent:nil.
    methodsFile size > 0 ifTrue:[
	properties at:#'methodsFile' put:methodsFile.
    ].
    files := pack at:'files' ifAbsent:nil.
    files notNil ifTrue:[
	properties at:#'files' put:files.
    ].

    "/ fetch class info
    "/
    "/ each entry consist of:
    "/    full-name-of-class
    "/    condition (optional)  - #unix / #win32 / #vms / #macos / #always / #never / #autoload
    "/    fileName (optional)
    (pack at:'classes' default:#()) do:[:info |
	|condKey className optionalFileName|

	condKey := #always.
	info isSymbol ifTrue:[
	    className := info.
	] ifFalse:[
	    className := info at:1.
	    info size > 1 ifTrue:[
		condKey := info at:2.
		info size > 2 ifTrue:[
		    optionalFileName := info at:3.
		]
	    ].
	].
	self 
	    addClass:className 
	    conditionForInclusion:condKey 
	    classFileName:optionalFileName
    ].

    "/ fetch methods info

    (pack at:'methods' default:#()) do:[:info |
	|condKey className methodName optionalFileName|

	condKey := #always.
	className := info at:1.
	methodName := info at:2.
	info size > 2 ifTrue:[
	    optionalFileName := info at:3.
	    info size > 3 ifTrue:[
		condKey := info at:4.
	    ]
	].
	self 
	    addMethod:methodName inClass:className 
	    conditionForInclusion:condKey 
	    fileName:optionalFileName
    ].

    self wasLoadedFromFile:true.

    "/ all remaining properties
    pack keysAndValuesDo:[:key :val |
	(key startsWith:'property.') ifTrue:[
	    self propertyAt:(key copyFrom:'property.' size+1) asSymbol put:val.
	]
    ].

    "
     Project current saveAsProjectFile.

     Project new loadFromProjectFile:'default.prj'

     Project new loadFromProjectFile:'../../libbasic/libbasic.prj'
    "

    "Modified: / 26.4.1999 / 23:21:33 / cg"
!

saveAsProjectFile
    "store all of my attributes into a projects file (.prj-file)
     (in the project-directory and named as <projectName>.prj)"

    |dir fn nm s|

    dir := self directory asFilename.
    nm := self libraryName.
    fn := dir construct:nm.
    fn := fn withSuffix:'prj'.

    fn exists ifTrue:[
	fn copyTo:(fn pathName , '.bak')
    ].
    s := fn writeStream.
    self saveAsProjectFileOn:s.
    s close.

    "
     Project current saveAsProjectFile
    "

    "Modified: / 26.4.1999 / 22:53:38 / cg"
!

saveAsProjectFileOn:aStream
    "save the project info in a format which is both usable for reload
     and somehow readable for humans.
     Actually, the format is the same as used for resources (i.e. key - value pairs)
     and the code below could be much simpler - if there where no humans to read it ..."

    |s coll first maxLen t defNS methodsFile|

    s := aStream.

    s nextPutLine:'; $Header' , '$'; nextPutLine:';'.
    s nextPutLine:'; Project saved ' , Smalltalk timeStamp; nextPutLine:';'.
    s nextPutAll:'; Be careful when editing - do not corrupt the files syntax.
; Lines starting with a semicolon are comment lines.
; Lines ending with a backslash are concatenated with the following line.
; Entries are key-value pairs, separated by whitespace;
; the value is a smalltalk literal expression.
'.

    s nextPutAll:'
;
; general:
;
'.

    s nextPutAll:'comment'. 
    s tab. s nextPutLine:(self comment storeString).

    s nextPutAll:'name'. 
    s tab. s nextPutLine:(name storeString).

    s nextPutAll:'type'. 
    s tab. s nextPutLine:(self type storeString).

    s nextPutAll:'package'. 
    s tab. s nextPutLine:(self package storeString).

    defNS := self defaultNameSpace.
    (defNS ~~ Smalltalk) ifTrue:[
	s nextPutAll:'nameSpace'. 
	s tab. s nextPutLine:(defNS name storeString).
    ].

    s nextPutAll:'
;
; repository:
;
'.
    s nextPutAll:'repository.module'. 
    s tab. s nextPutLine:(repositoryModule ? 'private') storeString.
    s nextPutAll:'repository.directory'. 
    s tab. s nextPutLine:(repositoryDirectory ? self package) storeString.
    s cr.

    (t := properties at:#'sourcesDirectory' ifAbsent:nil) notNil ifTrue:[
	s nextPutAll:'sources'. 
	s tab. s nextPutLine:(t storeString).
    ].

    first := true.
    properties keysAndValuesDo:[:key :val |
	(#(
	    comment
	    wasLoadedFromFile
	    targetconditions
	    classes
	    classInfo
	    methodInfo
	    prerequisiteClasses
	    files
	    sourcesDirectory
	    methodsFile
	    type
	    defaultNameSpace
	) includes:key) ifFalse:[
	    first ifTrue:[
		first := false.
		s nextPutAll:'
;
; properties:
;
'.
	    ].    
	    s nextPutAll:'property.'; nextPutAll:key. 
	    s tab. 
	    key == #defaultNameSpace ifTrue:[
		s nextPutLine:val name storeString.
	    ] ifFalse:[
		s nextPutLine:val storeString.
	    ]
	]
    ].
	
"/    coll := self subProjects.
"/    coll size > 0 ifTrue:[
"/        s nextPutLine:'[subprojects]'. 
"/        coll do:[:aSubProject |
"/            s tab. s nextPutLine:(aSubProject name soreString).
"/        ].
"/    ].

    s nextPutAll:'
;
; required packages:
;
'.
    s nextPutAll:'prerequisites'; tab.
    coll := self prerequisites.
    coll size = 0 ifTrue:[
	s nextPutLine:'#()'. 
    ] ifFalse:[    
	s nextPutLine:'#( \'. 
	coll do:[:aProjectOrProjectNameList |
	    |pName pPath|

	    aProjectOrProjectNameList isString ifTrue:[
		pName := aProjectOrProjectNameList.
	    ] ifFalse:[
		aProjectOrProjectNameList isArray ifTrue:[
		    pName := aProjectOrProjectNameList at:1.
		    pPath := aProjectOrProjectNameList at:2.
		] ifFalse:[
		    pName := aProjectOrProjectNameList name.
		    pPath := aProjectOrProjectNameList repositoryPath.    
		]
	    ].

	    pPath isNil ifTrue:[
		s tab. s nextPutAll:(pName storeString); nextPutLine:' \'.
	    ] ifFalse:[
		s tab. s nextPutAll:'( '.
		s nextPutAll:(pName storeString); space;
		  nextPutAll:(pPath storeString); nextPutLine:') \'.
	    ]
	].
	s nextPutLine:')'.
    ].

    s nextPutAll:'
;
; required classes:
;
'.
    s nextPutAll:'prerequisiteClasses'; tab.
    coll := self prerequisiteClasses.
    coll size = 0 ifTrue:[
	s nextPutLine:'#()'. 
    ] ifFalse:[    
	s nextPutLine:'#( \'. 
	coll do:[:aClassOrSymbol | |className|

	    (className := aClassOrSymbol) isSymbol ifFalse:[
		className := aClassOrSymbol name
	    ].
	    s tab. s nextPutAll:(className storeString); nextPutLine:' \'.
	].
	s nextPutLine:')'.
    ].

    s nextPutAll:'
;
; classes:
; (for each class, one line of the form: ( #''className'' [condition [fileName]] )
; (where fileName and condition are optional)
;
'.

    s nextPutAll:'classes'; tab.

    coll := self classInfo.
    coll size = 0 ifTrue:[
	s nextPutLine:'#()'. 
    ] ifFalse:[    
	s nextPutLine:'#( \'.
	"/ find the longest className (for layout only)
	
	maxLen := coll inject:0 into:[:maxSoFar :aClassInfo |
					|clsName|

					clsName := aClassInfo className.
					maxSoFar max:clsName storeString size
				     ].

	coll do:[:aClassInfo |
	    |clsName fileName cond|

	    clsName := aClassInfo className.
	    fileName := aClassInfo classFileName.
	    fileName = (clsName , '.st') ifTrue:[
		fileName := nil
	    ].
	    cond := (aClassInfo conditionForInclusion) ? #always.
	    s tab. s nextPutAll:'( '; 
		     nextPutAll:(clsName storeString paddedTo:maxLen).
	    (cond ~~ #always or:[fileName notNil]) ifTrue:[
		s tab; nextPutAll:cond storeString.
	    ].
	    fileName notNil ifTrue:[
		s tab; nextPutAll:fileName storeString.
	    ].
	    s nextPutLine:') \'.
	].
	s nextPutLine:')'.
    ].

    s nextPutAll:'
;
; methods (patches & extensions):
; (for each method, one line of the form: ( #''className'' #''methodName'' )
; (for metaclasses, #''name class'' is used)
;
'.

    s nextPutAll:'methods'; tab.

    coll := self methodInfo.
    coll size = 0 ifTrue:[
	s nextPutLine:'#()'. 
    ] ifFalse:[    
	s nextPutLine:'#( \'.
	"/ find the longest className (for layout only)

	maxLen := coll inject:0 into:[:maxSoFar :aMethodInfo |
					|clsName|

					clsName := aMethodInfo className.
					maxSoFar max:clsName storeString size
				     ].

	coll do:[:aMethodInfo |
	    |clsName mthdName fileName cond|

	    clsName := aMethodInfo className.
	    mthdName := aMethodInfo methodName.
	    s tab. s nextPutAll:'( '; 
		     nextPutAll:(clsName storeString paddedTo:maxLen); 
		     tab; nextPutAll:mthdName storeString.
	    s nextPutLine:') \'.
	].
	s nextPutLine:')'.
    ].

    methodsFile := properties at:#methodsFile ifAbsent:nil.
    methodsFile size > 0 ifTrue:[
	s cr; nextPutLine:';'; nextPutLine:'; methods above are stored in:'; nextPutLine:';'.
	s nextPutLine:';'.
	s nextPutAll:'methodsFile'; tab; nextPutLine:'''' , methodsFile , ''''.
    ].

    s nextPutAll:'
;
; files (for deployment):
;
'.

    s nextPutAll:'files'; tab.
    coll := properties at:#'files' ifAbsent:#().
    coll size = 0 ifTrue:[
	s nextPutLine:'#()'. 
    ] ifFalse:[    
	s nextPutLine:'#( \'. 
	coll do:[:aFileEntry |
	    s tab. s nextPutAll:(aFileEntry storeString); nextPutLine:' \'.
	].
	s nextPutLine:')'.
    ].

    "
     Project current saveOn:Transcript
    "

    "Modified: / 26.4.1999 / 23:24:12 / cg"
! !

!Project methodsFor:'maintenance'!

buildProject
    "invoke 'make' in the project directory"

    OperatingSystem executeCommand:('cd ' , self directory , ' ; make')
!

createLoadAllFile
    "creates a 'loadAll' file, which will load all classes
     of the project - this loadAll file is supposed to be located
     in the projects source directory."

    |d f out in topName classes classInfo numBad firstBad msg 
     methodsFile prerequisitePackages|

    classes := self classes.
    numBad := 0.
    firstBad := nil.

    classes := classes collect:[:clsOrSymbol |  |cls|
					clsOrSymbol isSymbol ifTrue:[
					    cls := Smalltalk at:clsOrSymbol.
					    cls isNil ifTrue:[
						numBad := numBad + 1.
						firstBad := firstBad ? clsOrSymbol.
					    ] ifFalse:[
						cls isLoaded ifFalse:[
						    cls autoLoad.
						    cls isLoaded ifFalse:[
							cls := nil
						    ]
						].
					    ].
					    cls.
					] ifFalse:[
					    clsOrSymbol
					]
			      ].

    numBad ~~ 0 ifTrue:[
	msg := 'Cannot generate ''loadAll''-file.\\'.
	msg := msg , 'Reason: Class ''' , firstBad asText allBold
		   , ''' is not loaded.'.
	numBad ~~ 1 ifTrue:[
	    msg := msg , '\(' , (numBad-1) printString , ' more unloaded classes were found)'
	].
	self warn:msg withCRs.
	^ self.
    ].

    "/ to not list private classes
    classes := classes select:[:cls | cls owningClass isNil].
    classes := classes copy topologicalSort:[:a :b | b isSubclassOf:a].

    Transcript showCR:'creating loadAll file'.

    d := directoryName asFilename.
    f := d construct:'loadAll'.
    f exists ifTrue:[
	f renameTo:(d construct:'loadAll.bak')
    ].
    out := f writeStream.
    out isNil ifTrue:[
	self warn:'cannot create loadAll'.
	(d construct:'loadAll.bak') renameTo:f.
	^ self
    ].

    out nextPutLine:'"/
"/ $' , 'Header' , '$'.
    out nextPutLine:'"/
"/ loadAll-file to fileIn code for: ' , self package.
    
    out nextPutAll:'"/
"/ Automatically generated from project definition.
"/ DO NOT MODIFY THIS fILE;
"/ modify the .prj file instead, and regenerate this file
"/ with the ProjectBrowser tool.
"/

"/
"/ Prerequisites:
"/'.
    prerequisitePackages := self prerequisitePackages 
				collect:[:entry |
				    |pName|

				    entry isString ifTrue:[
					pName := entry
				    ] ifFalse:[
					entry isArray ifTrue:[
					    pName := entry at:1
					] ifFalse:[
					    pName := entry name
					]
				    ]
				].

    prerequisitePackages size == 0 ifTrue:[
	out nextPutLine:''.
	out nextPutLine:'"/ Smalltalk loadPackage:''module:directory''.'.
	out nextPutLine:'"/ Smalltalk loadPackage:''....''.'.
    ] ifFalse:[
	out cr.
	prerequisitePackages do:[:packName |
	    out nextPutLine:'Smalltalk loadPackage:''' , packName , '''.'.
	]
    ].

    out nextPutAll:'!!

"{ package:''' , self package , ''' }"!!

|files|

''loading package ' , self package , ' ...'' infoPrintCR.

files := #(
.
'.

    classes do:[:cls |
	|clsInfo cond|

	clsInfo := self classInfoFor:cls.
	cond := clsInfo conditionForInclusion.
	(cond == #always or:[cond == #autoload]) ifTrue:[
	    out nextPutAll:'  '''.
	    cls nameWithoutNameSpacePrefix printOn:out.
	    out nextPutAll:'.st'''.
	    out cr.
	]
    ].
    (methodsFile := self propertyAt:#methodsFile) size > 0 ifTrue:[
	 out 
	    nextPutAll:'  ''';
	    nextPutAll:methodsFile;
	    nextPutAll:'''';
	    cr.
    ].

    out nextPutAll:') asOrderedCollection.

(Smalltalk loadClassLibraryIfAbsent:''' , self libraryName , ''') ifTrue:[
    |handle loaded|

    handle := ObjectFileLoader loadedObjectHandles 
		    detect:[:h | h package = ''' , self package , '''] ifNone:nil.
    handle ifNotNil:[
	loaded := Set new:(handle classes size).
	handle classes do:[:c| c isMeta ifFalse:[loaded add:c classFilename]].
'.
    
    methodsFile size > 0 ifTrue:[
	out nextPutLine:('        loaded add:''' , methodsFile , '''.').
    ].
    out nextPutAll:'        files := files asOrderedCollection select:[:f| (loaded includes:f) not].
    ].
].

files do:[:s |
    ''.'' infoPrint.
    Smalltalk fileIn:s.
].
'' '' infoPrintCR.
'' done (' , self package , ').'' infoPrintCR.
.'.

    out close
!

createMakefile
    "creates an initial Makefile from a Make.proto file.
     Requires the stmkmf script"

    |d top|

    Transcript showCR:'creating Makefile'.

    d := directoryName asFilename pathName.
    top := (d , '/' , (self findTopFrom:directoryName)) asFilename pathName.

    (OperatingSystem 
	executeCommand:('sh ' , top , '/rules/stmkmf')
	inDirectory:d) ifFalse:[
	self warn:'Could not execute stmkmf - no Makefile built.'
    ].

!

createProjectFiles
    "actually, creates all files to do a make in the project directory"

    directoryName asFilename exists ifFalse:[
	(self confirm:'create new projectDirectory: ' , directoryName) 
	    ifFalse:[^ self].
	OperatingSystem recursiveCreateDirectory:directoryName.
    ].
    self createMakefile.
    self createSourcefiles.
    self createProtoMakefile.
    (self propertyAt:#deliverLoadAllFile) == true ifTrue:[
	self createLoadAllFile
    ].
!

createProtoMakefile
    "creates a Make.proto file"

    |d f s type appName libName startUpClass startUpSelector
     topName classes numBad firstBad msg methodsFile|

    classes := self classes.
    numBad := 0.
    firstBad := nil.

    classes := classes collect:[:clsOrSymbol |  |cls|
					clsOrSymbol isSymbol ifTrue:[
					    cls := Smalltalk at:clsOrSymbol.
					    cls isNil ifTrue:[
						numBad := numBad + 1.
						firstBad := firstBad ? clsOrSymbol.
					    ] ifFalse:[
						cls isLoaded ifFalse:[
						    cls autoLoad.
						    cls isLoaded ifFalse:[
							cls := nil
						    ]
						].
					    ].
					    cls.
					] ifFalse:[
					    clsOrSymbol
					]
			      ].
    numBad ~~ 0 ifTrue:[
	msg := 'Cannot generate ''Make.proto''-file.\\'.
	msg := msg , 'Reason: Class ''' , firstBad asText allBold
		   , ''' is not loaded.'.
	numBad ~~ 1 ifTrue:[
	    msg := msg , '\(' , (numBad-1) printString , ' more unloaded classes were found)'
	].
	self warn:msg withCRs.
	^ self.
    ].

    "/ to not list private classes
    classes := classes select:[:cls | cls owningClass isNil].
    classes := classes copy topologicalSort:[:a :b | b isSubclassOf:a].

    topName := self findTopFrom:directoryName.


    Transcript showCR:'creating Make.proto'.

    d := directoryName asFilename.
    f := d construct:'Make.proto'.
    f exists ifTrue:[
	f renameTo:(d construct:'Make.proto.bak')
    ].
    s := f writeStream.
    s isNil ifTrue:[
	self warn:'cannot create prototype Makefile'.
	^ self
    ].
    s nextPutAll:'# $' , 'Header'. 
    s nextPutLine:'$'.
    s nextPutAll:'#
# -- Make.proto created from project at ' . 
    s nextPutAll:Smalltalk timeStamp. 
    s nextPutAll:'
#
# Warning: YOU SHOULD NOT MODIFY THIS FILE - MODIFY THE .prj FILE INSTEAD
# and let the ProjectBrowser recreate this file.
# once you modify this file, do not rerun
# stmkmp or recreate the Make.proto again - otherwise, your changes are lost.

# module and directory-in-module;
# these should correspond to the directory hierarchy
# location (otherwise, ST/X will have a hard time to
# find out the packages location from its packageID)
MODULE=' , self repositoryModule ? 'unknown' , '
MODULE_DIR=' , self repositoryDirectory ? 'unknown' , '

# default installation directory:
# (overwrite with ''make INSTALLTOP_DIR=... install'')
# the INSTALLBASE is imported from configurations... and usually
# defaults to something like /opt/smalltalk.
# (overwrite with ''make INSTALLBASE=... install'')
INSTALLTOP_DIR=$(INSTALLBASE)/packages/$(MODULE)/$(MODULE_DIR)
INSTALLLIB_DIR=$(INSTALLTOP_DIR)
INSTALLBIN_DIR=$(INSTALLTOP_DIR)

#
# position (of this package) in directory hierarchy:
# (must point to ST/X top directory, for tools and includes)
TOP=' , topName ,'

# subdirectories where targets are to be made:
SUBDIRS=

# subdirectories where Makefiles are to be made:
# (only define if different from SUBDIRS)
# ALLSUBDIRS=


# the name of your classLibrary:
# ********** REQUIRED: CHECK the next line ***
LIBNAME=' , self libraryName , '


# the next define suppresses installation of 
# the classes as autoloaded (i.e. not added to abbrev.stc). 
SUPPRESS_LOCAL_ABBREVS=1

# the package is stored as an ID in classes and methods
# to identify code belonging to this project.
# It also specifies the position in the source repository
# and directory tree, when packages are loaded by packageID.
# ********** REQUIRED: CHECK the next line ***
PACKAGE=$(MODULE):$(MODULE_DIR)

# Argument(s) to the stc compiler.
#  -H.         : create header files locally
#                (if removed, they will be created as common
#  -Pxxx       : defines the package
#  -Zxxx       : a prefix for variables within the classLib
#  -Dxxx       : defines passed to to CC for inline C-code
#  -Ixxx       : include path passed to CC for inline C-code
#  +optspace   : optimized for space
#  +optspace2  : optimized more for space
#  +optspace3  : optimized even more for space
#  +optinline  : generate inline code for some ST constructs
#  +inlineNew  : additionally inline new
#  +inlineMath : additionally inline some floatPnt math stuff
#
# ********** OPTIONAL: MODIFY the next line(s) ***
# STCLOCALOPTIMIZATIONS=+optinline +inlineNew
# STCLOCALOPTIMIZATIONS=+optspace3
STCLOCALOPTIMIZATIONS=+optspace3

# Argument(s) to the stc compiler.
#  -warn            : no warnings
#  -warnNonStandard : no warnings about ST/X extensions
#  -warnEOLComments : no warnings about EOL comment extension
#  -warnPrivacy     : no warnings about privateClass extension
#
# ********** OPTIONAL: MODIFY the next line(s) ***
# STCWARNINGS=-warn
# STCWARNINGS=-warnNonStandard
# STCWARNINGS=-warnEOLComments
STCWARNINGS='.
	(self propertyAt:#'make.stc.warnEOLComments') == false ifTrue:[
	    s nextPutAll:'-warnEOLComments '.
	].
	(self propertyAt:#'make.stc.warnNonStandard') == false ifTrue:[
	    s nextPutAll:'-warnNonStandard '.
	].
	s nextPutAll:((self propertyAt:#'make.stc.WARNINGOPTIONS') ? '') , '

# if your embedded C code requires any system includes, 
# add the path(es) here:, 
# ********** OPTIONAL: MODIFY the next lines ***
# LOCALINCLUDES=-Ifoo -Ibar
LOCALINCLUDES=' , ((self propertyAt:#'make.stc.LOCALINCLUDES') ? '') , '

# if you need any additional defines for embedded C code, 
# add them here:, 
# ********** OPTIONAL: MODIFY the next lines ***
# LOCALDEFINES=-Dfoo -Dbar -DDEBUG
LOCALDEFINES=' , ((self propertyAt:#'make.stc.LOCALDEFINES') ? '') , '

STCLOCALOPT=-I. $(STCLOCALOPTIMIZATIONS) $(STCWARNINGS) $(LOCALINCLUDES) $(LOCALDEFINES) -H. ''-P$(PACKAGE)'' ''-Z$(LIBNAME)'' $(COMMONSYMFLAG) $(INITCODESEPFLAG)

# ********** OPTIONAL: MODIFY the next line ***
# additional C-libraries that should be pre-linked with the class-objects
LD_OBJ_LIBS=' , ((self propertyAt:#'make.LD_OBJ_LIBS') ? '') , '

# ********** OPTIONAL: MODIFY the next line ***
# additional C targets or libraries should be added below
LOCAL_EXTRA_TARGETS=' , ((self propertyAt:#'make.LOCAL_EXTRA_TARGETS') ? '') , '

all:: preMake classLibRule postMake


'.

"/    type := #library.
"/    appName := 'app'.
"/    libName := 'lib'.
"/    startUpClass := 'Smalltalk'.
"/    startUpSelector := 'start'.
"/
"/    properties notNil ifTrue:[
"/        type := properties at:#projectType ifAbsent:type.
"/        appName := properties at:#applicationName ifAbsent:appName.
"/        startUpClass := properties at:#startupClass ifAbsent:startUpClass.
"/        startUpSelector := properties at:#startupSelector ifAbsent:startUpSelector.
"/    ].
"/
"/    s nextPutAll:'#
"/# define the name of the library to create
"/#
"/'.
"/    s nextPutAll:'LIBNAME=lib' , appName; cr; cr.
"/
"/    s nextPutAll:'#
"/# the target rule:
"/#
"/all::   abbrev.stc objs genClassList $(OBJTARGET)
"/
"/'.
"/
"/    type == #executable ifTrue:[
"/        s nextPutAll:'PROGS = ' , appName; cr.
"/        s nextPutAll:('STARTUP_CLASS=' , startUpClass); cr.
"/        s nextPutAll:'STARTUP_SELECTOR="' , startUpSelector; nextPutAll:'"'; cr.
"/    ].
"/
"/    s nextPutAll:'#
"/# define the object files that are to be created
"/#
"/'.
    s nextPutAll:'OBJS='.

    classes do:[:aClass |
	|abbrev clsInfo cond include|

	clsInfo := self classInfoFor:aClass.
	include := true.
	clsInfo notNil ifTrue:[
	    cond := clsInfo conditionForInclusion.
	    (self conditionForInclusionIsTrue:cond) ifFalse:[
		include := false.
	    ] ifTrue:[
		(self conditionForAutoloadIsTrue:cond) ifTrue:[
		    include := false
		]
	    ].
	].
	include ifTrue:[
	    s nextPutAll:' \'. s cr.
	    abbrev := Smalltalk fileNameForClass:aClass name.
	    s nextPutAll:'  '; nextPutAll:abbrev; nextPutAll:'.$(O)'.
	]
    ].
    (methodsFile := self propertyAt:#methodsFile) size > 0 ifTrue:[
	 s nextPutAll:' \'. s cr.
	 s nextPutAll:'  '; 
	   nextPutAll:(methodsFile asFilename withoutSuffix baseName);
	   nextPutAll:'.$(O)'.
    ].
    s cr.

    s nextPutAll:'
# add more install actions here
install::

# add more install actions for aux-files (resources) here
installAux::

# add more preMake actions here
preMake::

# add more postMake actions here
postMake:: cleanjunk

cleanjunk::

clean::
' , Character tab , '-rm -f *.o *.H

clobber::
' , Character tab , '-rm -f *.so *.dll

$(INSTALLBASE)::
' , Character tab , '@test -d $@ || mkdir $@

$(INSTALLBASE)/packages:: $(INSTALLBASE)
' , Character tab , '@test -d $@ || mkdir $@

$(INSTALLBASE)/packages/$(MODULE):: $(INSTALLBASE)/packages
' , Character tab , '@test -d $@ || mkdir $@

$(INSTALLBASE)/packages/$(MODULE)/osi:: $(INSTALLBASE)/packages/$(MODULE)
' , Character tab , '@test -d $@ || mkdir $@

$(INSTALLBASE)/packages/$(MODULE)/osi/asn1:: $(INSTALLBASE)/packages/$(MODULE)/osi
' , Character tab , '@test -d $@ || mkdir $@

$(INSTALLBASE)/packages/$(MODULE)/$(MODULE_DIR):: $(INSTALLBASE)/packages/$(MODULE)
' , Character tab , '@test -d $@ || mkdir $@

# BEGINMAKEDEPEND --- do not remove this line; make depend needs it
'.
"/    classes notNil ifTrue:[
"/        classes do:[:aClass |
"/            |abbrev|
"/
"/            abbrev := Smalltalk fileNameForClass:aClass name.
"/            s nextPutAll:abbrev; nextPutAll:'.$(O): '.
"/            s nextPutAll:abbrev; nextPutAll:'.st '.
"/            aClass allSuperclassesDo:[:superClass|
"/                s nextPutAll:(Smalltalk fileNameForClass:superClass name) , '.$(H) '.
"/            ].
"/            s nextPutAll:'$(STCHDR)';  cr.
"/        ].
"/    ].
    s nextPutAll:'# ENDMAKEDEPEND --- do not remove this line'.
    s cr; cr.

    type == #executable ifTrue:[
	s nextPutAll:'all:: $(PROGS)'; cr.

	s nextPutAll:appName.
	s nextPutAll:':: main.o classList.o $(OBJS)'; cr.
	s tab;      nextPutAll:'$(LD) $(ST_LDFLAG) $(LDFLAGS) -o ';
		    nextPutAll:appName;
		    nextPutAll:' \'; cr.
	s tab; tab; nextPutAll:'$(CRT0) main.$(O) classList.$(O) $(OBJS) $(EXTRA_OBJ) $(LIBOBJS) \'; cr.
	s tab; tab; nextPutAll:'$(LIBRUNDIR)/hidata.o $(LIBRUN) \'; cr.
	s tab; tab; nextPutAll:'$(MATHLIB) $(EXTRALIBS) -lXext $(SYSLIBS) $(OTHERLIBS) $(CRTN)'; cr.
    ].

    s close

    "Modified: / 23.3.1999 / 14:20:09 / cg"
!

createSourcefiles
    "creates all Smalltalk-source files in the project directory"

    |classes methods methodClasses dir stream|

    dir := self directory asFilename.
    Transcript showCR:'creating sources in ' , dir pathName , ' ...'; endEntry.

    classes := self classes.
    classes isNil ifTrue:[
	self warn:'no classes in current project'
    ].

    classes notNil ifTrue:[
	classes do:[:aClass |
	    aClass isLoaded ifFalse:[
		aClass autoload.
	    ].
	].
	"/ to not list private classes
	classes := classes select:[:cls | cls owningClass isNil].
	classes := classes topologicalSort:[:a :b | a isSubclassOf:b].

	classes do:[:aClass |
	    Transcript show:' ... '; showCR:aClass name, '.st'; endEntry.
	    aClass fileOutIn:dir
	]
    ].

    methods := self individualMethods.
    methods notNil ifTrue:[
	methods := methods asIdentitySet.
	"
	 get classes ...
	"
	methodClasses := IdentitySet new.
	methods do:[:m | 
			|mCls|

			mCls := m containingClass.
			mCls isMeta ifTrue:[
			    mCls := mCls soleInstance.
			].
			methodClasses add:mCls].
	"
	 fileOut by class
	"
	methodClasses do:[:cls |
	    stream := (self directory asFilename construct:(cls name , '.chg')) writeStream.

	    Transcript show:' ... '; showCR:cls name, '.chg'; endEntry.
	    methods do:[:m |
		|mCls|

		mCls := m containingClass.
		(mCls == cls or:[mCls == cls class]) ifTrue:[
		    mCls fileOutMethod:m on:stream.
		].
		stream cr.
	    ].
	    stream close.
	].
    ].

    "Modified: 1.11.1996 / 16:37:15 / cg"
    "Modified: 18.9.1997 / 18:50:34 / stefan"
!

findTopFrom:directoryName
    "returns the relative path from directoryName to the TOP
     directory."

    |topName relParent foundTop|

    "/ find TOP
    relParent := '..'.
    foundTop := false.
    [foundTop] whileFalse:[
	topName := directoryName asFilename construct:relParent.
	topName isRootDirectory ifTrue:[
	    self warn:'could not find TOP; assume absoulte path to TOP'.
	    ^ nil.
"/            topName := '/usr/local/lib/smalltalk'.
"/            foundTop := true.
	] ifFalse:[
	    ((topName construct:'stx') construct:'configurations') exists ifTrue:[
		((topName construct:'stx') construct:'include') exists ifTrue:[
		    ^ relParent , '/stx'.
		]
	    ].
	    (topName construct:'configurations') exists ifTrue:[
		(topName construct:'include') exists ifTrue:[
		    ^ relParent 
		]
	    ].
	    relParent := relParent , '/..'
	]
    ].
    ^ topName pathName
! !

!Project methodsFor:'printing & storing'!

displayString
    ^ super displayString , '(''' , (name ? '<unnamed>') , ''')'
! !

!Project methodsFor:'properties'!

addClass:classOrClassName classFileName:fileName
    "add a class to the project"

    self
	addClass:classOrClassName 
	conditionForInclusion:#always 
	classFileName:fileName

!

addClass:classOrClassName conditionForInclusion:conditionBlock classFileName:fileName
    "add a class to the project"

    |i clsName|

    (clsName := classOrClassName) isBehavior ifTrue:[
	clsName := classOrClassName name
    ].

    i := ClassInfo new.
    i className:clsName.
    i classFileName:fileName.
    i conditionForInclusion:conditionBlock.
    self addClassInfo:i
!

addClassInfo:newInfo
    "add a class info to the project"

    |infoCollection index nm prefix|

    (infoCollection := self classInfo) isNil ifTrue:[
	self classInfo:(infoCollection := OrderedCollection new).
    ].

    index := infoCollection findFirst:[:i | |nm1 nm2|
					nm1 := i className.
					nm2 := newInfo className.
					nm1 = nm2
				      ].
    index ~~ 0 ifTrue:[
	infoCollection at:index put:newInfo
    ] ifFalse:[
	infoCollection add:newInfo
    ]
!

addMethod:methodName inClass:classOrClassName conditionForInclusion:condition fileName:optionalFileName
    "add a method to the project"

    |i|

    i := MethodInfo new.
    i className:classOrClassName.
    i methodName:methodName.
    i fileName:optionalFileName.
    i conditionForInclusion:condition.
    self addMethodInfo:i
!

addMethod:methodOrSelector inClass:aClassOrClassName fileName:fileName
    "add an individual method to the project"

    |i|

    i := MethodInfo new.
    i methodName:methodOrSelector.
    i className:aClassOrClassName.
    i fileName:fileName.
    self addMethodInfo:i
!

addMethodInfo:newInfo
    "add a method info to the project"

    |infoCollection index nm prefix|

    (infoCollection := self methodInfo) isNil ifTrue:[
	self methodInfo:(infoCollection := OrderedCollection new).
    ].

    index := infoCollection findFirst:[:i | |cnm1 cnm2|
					cnm1 := i className.
					cnm2 := newInfo className.
					(cnm1 includes:$:) ifFalse:[
					    cnm1 := self defaultNameSpace name , '::' , cnm1
					].
					(cnm2 includes:$:) ifFalse:[
					    cnm2 := self defaultNameSpace name , '::' , cnm2
					].
					cnm1 = cnm2 and:[i methodName = newInfo methodName]
				      ].
    "/ strip off nameSpace prefix, if its the same as
    "/ the default ...

    nm := newInfo className.
    prefix := self defaultNameSpace name , '::'.
    (nm startsWith:prefix) ifTrue:[
	nm := nm copyFrom:(prefix size + 1).
	newInfo className:nm asSymbol.
    ].
    index ~~ 0 ifTrue:[
	infoCollection at:index put:newInfo
    ] ifFalse:[
	infoCollection add:newInfo
    ]
!

classInfo:aClassInfoCollection
    "set the class info of the project"

    self propertyAt:#classInfo put:aClassInfoCollection

!

comment
    "return the comment of the project"

    properties isNil ifTrue:[^ ''].
    ^ properties at:#comment ifAbsent:''
!

comment:aString
    "set the projects comment"

    self propertyAt:#comment put:aString

!

documentationURL
    "return the documentation-URL of the project"

    properties isNil ifTrue:[^ nil].
    ^ properties at:#documentationURL ifAbsent:nil
!

documentationURL:anURLString
    "set the projects documentation-URL"

    self propertyAt:#documentationURL put:anURLString

!

methodInfo:aMethodInfoCollection
    "set the method info of the project"

    self propertyAt:#methodInfo put:aMethodInfoCollection

!

properties
    "return the property dictionary"

    ^ properties
!

properties:p
    "set the property dictionary"

    properties := p
!

propertyAt:aKey
    "return a property; the key is a symbol"

    properties isNil ifTrue:[^ nil].
    ^ properties at:aKey ifAbsent:nil.
!

propertyAt:aKey put:aValue
    "set a property; the key is a symbol"

    |oldValue|

    oldValue := self propertyAt:aKey.

    properties isNil ifTrue:[
	properties := IdentityDictionary new
    ].
    properties at:aKey put:aValue.

    oldValue ~~ aValue ifTrue:[
	self changed:aKey.
    ].

    "Created: / 23.3.1999 / 14:21:11 / cg"
!

removeClass:classOrClassName
    "remove a class from the project"

    |infoCollection index className|

    (infoCollection := self classInfo) isNil ifTrue:[^ self].

    (className := classOrClassName) isBehavior ifTrue:[
	className := classOrClassName name
    ].

    index := infoCollection findFirst:[:i | i className = className.].
    index ~~ 0 ifTrue:[
	infoCollection removeIndex:index
    ]
!

type
    "return the type of project (one of #application, #library, #smalltalk)"

    properties isNil ifTrue:[^ #application].
    ^ properties at:#type ifAbsent:#application
!

type:aSymbol
    "set the projects type (one of #application, #library, #smalltalk)"

    |sym|

    (sym := aSymbol) == #classLibrary ifTrue:[
	sym := #library
    ].
    (#(application library extension smalltalk) includes:sym) ifFalse:[
	self warn:'invalid project type'.
	^ self
    ].
    self propertyAt:#type put:sym
!

wasLoadedFromFile
    "return true, if the project was loaded from a file"

    properties isNil ifTrue:[^ nil].
    ^ properties at:#wasLoadedFromFile ifAbsent:false

    "Modified: / 23.3.1999 / 13:59:32 / cg"
    "Created: / 23.3.1999 / 14:21:06 / cg"
!

wasLoadedFromFile:aBoolean
    "set/clear the flag stating that the project was loaded from a file"

    self propertyAt:#wasLoadedFromFile put:aBoolean
! !

!Project methodsFor:'queries'!

areAllClassesLoaded
    "return true, if all classes of the package are loaded
     (i.e. there are no autoloaded stubs present)"

    |classes|

    classes := self classes.
    classes isNil ifTrue:[^ isLoaded ? false].
    classes do:[:aClass |
	aClass isLoaded ifFalse:[^ false].
    ].
    ^ true

!

classInfo
    "return a classInfo collection of classes belonging to that project"

    |classInfo classes|

    properties notNil ifTrue:[
	classInfo := properties at:#classInfo ifAbsent:nil.
	classInfo notNil ifTrue:[^ classInfo].
    ].

    classes := self classes.
    classes size == 0 ifTrue:[
	classInfo := OrderedCollection new
    ] ifFalse:[
	classInfo := classes asOrderedCollection
			collect:[:class |
			    |i fn|

			    i := ClassInfo new.
			    i conditionForInclusion:#always.
			    i className:class name.
			    fn := class classFilename ? ((Smalltalk fileNameForClass:class) , '.st').
			    i classFileName:fn.
			    i
			]
    ].
    self propertyAt:#classInfo put:classInfo.
    ^ classInfo

    "Modified: 4.1.1997 / 16:51:18 / cg"
!

classInfoFor:aClassOrClassName
    "return a classInfo for a particular class"

    |classInfo clsName index|

    (classInfo := self classInfo) isNil ifTrue:[^ nil].
    (clsName := aClassOrClassName) isBehavior ifTrue:[
	clsName := aClassOrClassName name
    ].
    ^ classInfo detect:[:i | i className = clsName] ifNone:nil.


    "Modified: 4.1.1997 / 16:51:18 / cg"
!

classes
    "return a collection of classes belonging to that project.
     This excludes any private classes."

    |classes classInfo|

    properties notNil ifTrue:[
	classInfo := properties at:#classInfo ifAbsent:nil.
	classInfo notNil ifTrue:[
	    classes := classInfo collect:[:i | i className]
	] ifFalse:[
	    classes := properties at:#classes ifAbsent:nil
	]
    ].

    classes isNil ifTrue:[
	classes := OrderedCollection new.
	Smalltalk 
	    allClassesDo:[:aClass |
		(true "aClass owningClass isNil"
		and:[aClass isMeta not
		and:[aClass package = packageName
		and:[aClass isNamespace not or:[aClass == Smalltalk]]]]) ifTrue:[
		    classes add:aClass
		]
	    ].
	classes isEmpty ifTrue:[^ nil].
    ].
    ^ classes

    "Modified: 4.1.1997 / 16:51:18 / cg"
!

individualMethods
    "return a collection of individual methods belonging to that project,
     only methods are returned which are not contained in the
     projects class set."


"/
"/ obsoleted by #methods ...
"/
    |classes methods|

    classes := self classes.
    classes notNil ifTrue:[
	classes := classes asIdentitySet.
    ] ifFalse:[
	classes := #()
    ].

    methods := IdentitySet new.
    Smalltalk allBehaviorsDo:[:cls |
	|classToCheck|

	classToCheck := cls.
"/        cls isPrivate ifTrue:[
"/            classToCheck := cls topOwningClass
"/        ].
	(classes isNil 
	or:[((classes includes:classToCheck) 
	    or:[classes includes:classToCheck name]) not]) ifTrue:[
	    cls methodDictionary do:[:m |
		m package = packageName ifTrue:[
		    methods add:m
		]
	    ].
	    cls class methodDictionary do:[:m |
		m package = packageName ifTrue:[
		    methods add:m
		]
	    ].
	]
    ].
    ^ methods asArray

    "
     Project current classes
     Project current individualMethods
    "

    "Modified: 7.6.1996 / 09:16:25 / stefan"
!

isLoaded
    "return true, if all of this project-package has been loaded
     into the system"

    |binaryModule cls|

    isLoaded notNil ifTrue:[^ isLoaded].

    "/ check for loaded class-library - assume loaded if present.
    binaryModule := ObjectMemory binaryModuleInfo detect:[:i | i package = self package] ifNone:nil.
    binaryModule notNil ifTrue:[
        ^ true
    ].

    "/ check for all classes ...
    self classes do:[:aClassOrClassName |
        aClassOrClassName isBehavior ifFalse:[
            aClassOrClassName isSymbol ifTrue:[
                (cls := Smalltalk at:aClassOrClassName) isNil ifTrue:[
                    ^ false
                ].
                cls isBehavior ifFalse:[^ false].
                cls isLoaded ifFalse:[^ false].
            ] ifFalse:[
                self halt.
                ^ false
            ]
        ]
    ].

    "/ check for all patches & extensions ...
    self methods do:[:aMethodInfo |
        aMethodInfo theMethod isNil ifTrue:[
            ^ false
        ].
    ].

    ^ true

    "
     (Project projectWithId:#'stx:libbasic') isLoaded
     (Project projectWithId:#'stx:goodies/persistency') isLoaded 
    "

    "Modified: / 26.9.1999 / 13:39:55 / cg"
!

methodInfo
    "return a methodInfo collection of methods belonging to that project"

    |methodInfo methods|

    properties notNil ifTrue:[
	methodInfo := properties at:#methodInfo ifAbsent:nil.
	methodInfo notNil ifTrue:[^ methodInfo].
    ].

    methods := self methods.
    methods size == 0 ifTrue:[
	methodInfo := OrderedCollection new
    ] ifFalse:[
	methodInfo := methods asOrderedCollection
			collect:[:mthd |
			    |i fn who className selector|

			    mthd isMethod ifTrue:[   
				who := mthd who.
				className := who methodClass name.
				selector := who methodSelector.
				i := MethodInfo new.
				i conditionForInclusion:#always.
				i className:className.
				i methodName:className.
				fn := mthd sourceFilename.
				i fileName:fn.
				i
			    ] ifFalse:[
				mthd "/ already a methodInfo
			    ]
			]
    ].
    self propertyAt:#methodInfo put:methodInfo.
    ^ methodInfo

    "Modified: 4.1.1997 / 16:51:18 / cg"
!

methods
    "return a collection of methods belonging to that project.
     This excludes any methods which are already in my class-set."

    |methods methodsInfo|

    properties notNil ifTrue:[
	methodsInfo := properties at:#methodsInfo ifAbsent:nil.
	methodsInfo notNil ifTrue:[
	    methods := methodsInfo collect:[:i | i className]
	] ifFalse:[
	    methods := properties at:#methods ifAbsent:nil
	]
    ].

    methods isNil ifTrue:[
	methods := OrderedCollection new.
	Smalltalk allClassesDo:[:aClass |
	    (true "aClass owningClass isNil"
	    and:[aClass isMeta not
	    and:[aClass package ~= packageName
	    and:[aClass isNamespace not or:[aClass == Smalltalk]]]]) ifTrue:[
	    
		aClass methodDictionary keysAndValuesDo:[:sel :mthd |
		    mthd package = packageName ifTrue:[
			methods add:(MethodInfo new
					className:aClass name;
					methodName:sel;
					yourself)
		    ].
		].
		aClass class methodDictionary keysAndValuesDo:[:sel :mthd |
		    mthd package = packageName ifTrue:[
			methods add:(MethodInfo new
					className:(aClass name , ' class');
					methodName:sel;
					yourself)
		    ].
		]
	    ]
	].
	methods isEmpty ifTrue:[^ #()].
    ].
    ^ methods

    "Modified: 4.1.1997 / 16:51:18 / cg"
! !

!Project methodsFor:'specifications'!

readSpec
    |s chunk fileName|

    fileName := (properties at:#directoryName) asFilename construct:'.project'.
    s := fileName readStream.
    s isNil ifTrue:[^ self].
    [s atEnd] whileFalse:[
	chunk := s nextChunk.
	Compiler evaluate:chunk receiver:properties notifying:nil
    ].
    s close.

    "(Project new directory:'../projects/Clock') readSpec"
!

saveSpec
    |f d s|

    d := (properties at:#directoryName) asFilename.
    d exists ifFalse:[
	self error:'directory does not exist' mayProceed:true.
	^ self
    ].
    f := d construct:'.project'.
    s := f writeStream.
    s isNil ifTrue:[^ self].
    properties associationsDo:[:aProp |
	(aProp == #directoryName) ifFalse:[
	    s nextChunkPut:('self at:' , aProp key storeString, 
			       ' put:' , aProp value storeString).
	    s cr
	]
    ].
    s close

    "((Project new directory:'../projects/Clock') readSpec
	 directory:'../projects/xxx') saveSpec"
! !

!Project methodsFor:'views'!

addView:aView
    "add a view to this projects set of views"

    views notNil ifTrue:[views add:aView]

    "Modified: 14.2.1997 / 15:36:51 / cg"
!

destroyViews
    "destroy all views of this project"

    views notNil ifTrue:[
	views do:[:aView |
	    aView notNil ifTrue:[aView destroy]
	]
    ].
!

hideViews
    "hide all views of this project"

    views notNil ifTrue:[
	views do:[:aView |
	    aView notNil ifTrue:[aView unmap]
	]
    ].

    "Modified: 3.5.1996 / 23:48:51 / stefan"
!

removeView:aView
    "remove a view from this projects set of views"

    views notNil ifTrue:[
	views remove:aView ifAbsent:nil
    ]

    "Modified: 14.2.1997 / 15:37:20 / cg"
!

showViews
    "show all views of this project"

    views notNil ifTrue:[
	views do:[:aView |
	    aView notNil ifTrue:[aView remap]
	]
    ].

    "Modified: 3.5.1996 / 23:59:10 / stefan"
    "Modified: 14.2.1997 / 15:38:47 / cg"
! !

!Project::MethodInfo methodsFor:'accessing'!

className
    "return the value of the instance variable 'className' (automatically generated)"

    ^ className!

className:something
    "set the value of the instance variable 'className' (automatically generated)"

    className := something.!

conditionForInclusion
    "return the value of the instance variable 'conditionForInclusion' (automatically generated)"

    ^ conditionForInclusion!

conditionForInclusion:something
    "set the value of the instance variable 'conditionForInclusion' (automatically generated)"

    conditionForInclusion := something.!

fileName
    "return the value of the instance variable 'fileName' (automatically generated)"

    ^ fileName!

fileName:something
    "set the value of the instance variable 'fileName' (automatically generated)"

    fileName := something.!

methodName
    "return the value of the instance variable 'methodName' (automatically generated)"

    ^ methodName!

methodName:something
    "set the value of the instance variable 'methodName' (automatically generated)"

    methodName := something.! !

!Project::MethodInfo methodsFor:'printing & storing'!

displayString
    ^ 'MethodInfo: ' , className , ' ' , methodName


! !

!Project::MethodInfo methodsFor:'queries'!

method
    self obsoleteMethodWarning.
    ^ self theMethod.

    "Modified: / 26.9.1999 / 13:40:16 / cg"
!

theMethod
    |cls|

    cls := Smalltalk classNamed:className.
    cls isNil ifTrue:[ ^ nil].
    ^ cls compiledMethodAt:methodName asSymbol.

    "Created: / 26.9.1999 / 13:39:07 / cg"
! !

!Project::ClassInfo methodsFor:'accessing'!

classFileName
    "return the value of the instance variable 'classFileName' (automatically generated)"

    ^ classFileName!

classFileName:something
    "set the value of the instance variable 'classFileName' (automatically generated)"

    classFileName := something.!

className
    "return the value of the instance variable 'className' (automatically generated)"

    ^ className!

className:something
    "set the value of the instance variable 'className' (automatically generated)"

    className := something.!

conditionForInclusion
    "return the value of the instance variable 'conditionForInclusion' (automatically generated)"

    ^ conditionForInclusion!

conditionForInclusion:something
    "set the value of the instance variable 'conditionForInclusion' (automatically generated)"

    conditionForInclusion := something.! !

!Project::ClassInfo methodsFor:'printing & storing'!

displayString
    ^ 'ClassInfo: ' , className
! !

!Project::ClassInfo methodsFor:'queries'!

theClass
    |cls|

    cls := Smalltalk classNamed:className.
    cls isNil ifTrue:[ ^ nil].
    ^ cls

    "Created: / 26.9.1999 / 13:39:00 / cg"
! !

!Project class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic/Project.st,v 1.112 1999-09-26 13:29:45 cg Exp $'
! !
Project initialize!
